//------------------------------------------------------------------
/*
    
Main.c

Top level program file.

*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <globals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include "propeller-icc.h"

//------------------------------------------------------------------
// Protoypes of external routines
void APD_Control(void);                                        // Controls.c
void Tex_Control(void);                                        // Controls.c
void User_Action(void);                                        // Serial.c
void Unprocessed_Display(void);                        // Serial.c
void Fast_Display(void);                                    // Serial.c
void Control_Display(void);                                // Serial.c
void Distances_Display(void);                            // Serial.c
void Settings_Display(void);                            // Serial.c
void Print_Line(int length);                            // Serial.c
int SPI(void);                                                        // SPI.c
void Initialise_Hardware(void);                        // Utilities.c
void Initialise_Parameters(void);                    // Utilities.c
void BlinkLED(void);                                            // Utilities.c
void Flash(int pin);                                            // Utilities.c
void Convert_to_Distance(void);                        // Utilities.c

//------------------------------------------------------------------ START
void main()
    {
    Initialise_Hardware();                                    // Configure the Propeller hardware
    Initialise_Parameters();                                // Initialise the software parameters
    BlinkLED();                                                            // Intro light and sound show
    printf("\rDS00 LRF #%05d\r", serial_number);    // Intro on terminal    
    Print_Line(15);
    if(int_settings[DISPLAY_MODE] == 's')        // If the settings display is selected then refresh the screen
                Settings_Display();

    while (1)                                                                // Main loop
        {
        if(rx_flag == READY)                                    // Check for key press on the USB port (flag set by different cog)
            {
            Flash(BUZZER);                                            // Enunciate using the buzzer
            User_Action();                                            // Take action if a key pressed
            rx_flag = -1;                                                // Clear the key press flag when the user action is completed
            putchar('\r');                                            // New line
            if(int_settings[DISPLAY_MODE] == 's')    // If the settings display is selected then refresh the screen
                Settings_Display();
            }
        
        if(SPI() == READY)                                        // Check for result from DS00VQ100 chip
            {                                                                        // and store results ready for further action
            Convert_to_Distance();                            // Convert chip data into meters and apply corrections
            APD_Control();                                            // Control the APD noise
            Tex_Control();                                            // Control the time expansion
            
            switch (int_settings[DISPLAY_MODE])    // Check which display mode is active
        {
        case 'c':
          Control_Display();                            // Show values associated with the control loops
        break;
        case 'd':
          Distances_Display();                        // Show intermediate distance values
        break;
        case 'u':
          Unprocessed_Display();                    // Show data directly from DS00VQ100 chip
        break;
                case 's':                                                    // Show the settings display
                break;                                                        
        default:                                                    // 'f'
          Fast_Display();                                    // Show the final distance result only
        break;
        }
            }
        }
    }
/*
-----------------------------------------------------------------------------------------------------------
This work is licensed under the Creative Commons Attribution 3.0 Unported License.
To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/
or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
-----------------------------------------------------------------------------------------------------------
*/        
//------------------------------------------------------------------ END